import torch
from diffusers import EulerAncestralDiscreteScheduler, StableDiffusionPipeline
from lora_diffusion import patch_pipe, tune_lora_scale


def infer(prompt):
    model_id = "runwayml/stable-diffusion-v1-5"

    pipe = StableDiffusionPipeline.from_pretrained(model_id, torch_dtype=torch.float16).to("cuda")
    pipe.scheduler = EulerAncestralDiscreteScheduler.from_config(pipe.scheduler.config)

    patch_pipe(
        pipe,
        "/home/engineering/output/final_lora.safetensors",
        patch_text=True,
        patch_ti=True,
        patch_unet=True,
    )

    tune_lora_scale(pipe.unet, 1.00)
    tune_lora_scale(pipe.text_encoder, 1.00)

    torch.manual_seed(0)
    image = pipe(prompt, num_inference_steps=50, guidance_scale=7).images[0]
    image.save("/home/engineering/samples/output.jpg")
    image


if __name__ == "__main__":
    # get arguments from command line
    import sys

    prompt = sys.argv[1]

    infer(prompt)



import argparse
import os
import shutil
import subprocess

import requests


def parse_args():
    parser = argparse.ArgumentParser(description="Run Rubbrband ControlNet training script")
    parser.add_argument("dataset_dir", help="Path to directory containing training data")

    return parser.parse_args()


def main(**kwargs):
    script_dir = os.path.dirname(os.path.abspath(__file__))

    if not os.path.isfile(os.path.join(script_dir, "v1-5-pruned.ckpt")):
        url = "https://huggingface.co/runwayml/stable-diffusion-v1-5/resolve/main/v1-5-pruned.ckpt"
        response = requests.get(url)
        with open(os.path.join(script_dir, "v1-5-pruned.ckpt"), "wb") as f:
            f.write(response.content)

    if shutil.which("nvidia-smi"):
        gpu_arg = "--gpus all"
    else:
        gpu_arg = ""

    if "rb-control" in subprocess.check_output('docker ps -a --format "{{.Names}}"').decode("utf-8"):
        subprocess.call("docker stop rb-control")
        subprocess.call("docker rm rb-control")

    volumes = (
        f"-v {os.path.join(script_dir, 'v1-5-pruned.ckpt')}:/home/engineering/ControlNet/models/v1-5-pruned.ckpt "
        f"-v {os.path.abspath(kwargs['dataset_dir'])}:/home/engineering/ControlNet/training/fill50k"
    )
    subprocess.call(f"docker run --name rb-control {gpu_arg} -it -d {volumes} -d rubbrband/control:latest")

    conda_cmd = (
        "conda run --no-capture-output -n control"
        "python tool_add_control.py ./models/v1-5-pruned.ckpt ./models/control_sd15_ini.ckpt &&"
        "conda run --no-capture-output -n control python tutorial_train.py"
    )

    subprocess.call(f"docker exec -it rb-control /bin/bash -c '{' '.join(conda_cmd)}'", shell=True)


if __name__ == "__main__":
    main()



import docker
import typer

from rubbrband.clients.docker_client import pull_image_handler
from rubbrband.controllers.models.sd_webui.web import main as web_sd_webui

db = {}
client = None
app = typer.Typer(no_args_is_help=True)


@app.callback()
def main():
    """
    Start a webui for a MODEL :robot:

    Example: rubbrband web sd-webui
    """


@app.command(rich_help_panel="Models :robot:", help="Webui for stable diffusion models")
def sd_webui(
    ctx: typer.Context,
):
    web(ctx, "sd-webui")


def handle_model_web(params: dict, model: str):
    """Handle which model to run."""
    if model == "sd-webui":
        web_sd_webui()
    else:
        typer.echo(f"Model {model} not found")


# '''name''' corresponds to the name column in db.csv
# the option '''-d''' or '''--dataset_dir''' is the path to the dataset directory
# this directory gets mounted to the container at /home/engineering/data
def web(ctx: typer.Context, model: str):
    """Entrypoint for training a model."""
    if model not in db:
        typer.echo("Model not found")
        return
    image_name = f"rubbrband/{model}"
    container_name = f"rb-{model}"
    pull_image_handler(image_name)

    try:
        container = client.containers.get(container_name)

        # stop and remove container if it is already running

        if container.status == "running":
            container.stop()
            container.remove()
        container = client.containers.get(container_name)

    except docker.errors.NotFound:
        pass

    handle_model_web(ctx.params, model)


if __name__ == "__main__":
    app()
